@@ -1,4 +1,5 @@
 
+#define R_NO_REMAP
 
 #include <R.h>
 #include <Rinternals.h>
@@ -11,18 +12,7 @@
 
 
 #include "spng.h"
-#include "hash-color.h"
-
-//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
-// Hashed colour lookup from 'hash-color.c'
-//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
-extern char *col_name[];
-extern uint8_t col_int[][4];
-
-//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
-// Convert a hex digit to a nibble
-//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
-#define hex2nibble(x) ( (((x) & 0xf) + ((x) >> 6) + ((x >> 6) << 3)) & 0xf )
+#include "colorfast.h"
 
 
 //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
@@ -45,11 +35,11 @@ SEXP write_png_core_(void *image, size_t nbytes, uint32_t width, uint32_t height
   //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
   // Options
   //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
-  int use_filter        = asLogical(use_filter_);
-  int compression_level = asInteger(compression_level_);
+  int use_filter        = Rf_asLogical(use_filter_);
+  int compression_level = Rf_asInteger(compression_level_);
   if (compression_level < -1 || compression_level > 9) {
     if (free_image_on_error) free(image);
-    error("Invalid compression level. Must be in range [0, 9] not %i", compression_level);
+    Rf_error("Invalid compression level. Must be in range [0, 9] not %i", compression_level);
   }
   
   //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
@@ -63,7 +53,7 @@ SEXP write_png_core_(void *image, size_t nbytes, uint32_t width, uint32_t height
   //   spng_set_png_file() or spng_set_png_stream() 
   //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
   FILE *fp = NULL;
-  if (isNull(file_)) {
+  if (Rf_isNull(file_)) {
     spng_set_option(ctx, SPNG_ENCODE_TO_BUFFER, 1);
   } else {
     const char *filename = R_ExpandFileName(CHAR(STRING_ELT(file_, 0)));
@@ -71,14 +61,14 @@ SEXP write_png_core_(void *image, size_t nbytes, uint32_t width, uint32_t height
     if (fp == NULL) {
       if (free_image_on_error) free(image);
       spng_ctx_free(ctx);
-      error("Couldn't open file: %s", filename);
+      Rf_error("Couldn't open file: %s", filename);
     }
     err = spng_set_png_file(ctx, fp); 
     if (err) {
       fclose(fp);
       if (free_image_on_error) free(image);
       spng_ctx_free(ctx);
-      error("Couldn't set file for output: %s", filename);
+      Rf_error("Couldn't set file for output: %s", filename);
     }
   } 
   
@@ -114,54 +104,31 @@ SEXP write_png_core_(void *image, size_t nbytes, uint32_t width, uint32_t height
   //   SPNG_COLOR_TYPE_GRAYSCALE_ALPHA = 4,
   //   SPNG_COLOR_TYPE_TRUECOLOR_ALPHA = 6
   //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
-  if (!isNull(trns_)) {
+  if (!Rf_isNull(trns_)) {
     struct spng_trns trns;
     int has_trns = 0;
     if (color_type == SPNG_COLOR_TYPE_GRAYSCALE) {
       // Rprintf("trns given and valid for image type GRAYSCALE\n");
-      trns.gray = (uint16_t)(asInteger(trns_));
+      trns.gray = (uint16_t)(Rf_asInteger(trns_));
       has_trns = 1;
     } else if (color_type == SPNG_COLOR_TYPE_TRUECOLOR) {
       // Rprintf("trns given and valid for image type RGB\n");
       if (TYPEOF(trns_) == STRSXP) {
         // Rprintf("Hex color for trns\n");
         const char *col = CHAR(STRING_ELT(trns_, 0));
-        if (col[0] == '#') {
-          switch (strlen(col)) {
-          case 9:
-          case 7:
-            trns.red   = (uint16_t)( (hex2nibble(col[1]) << 4) + hex2nibble(col[2]) ); // R
-            trns.green = (uint16_t)( (hex2nibble(col[3]) << 4) + hex2nibble(col[4]) ); // G
-            trns.blue  = (uint16_t)( (hex2nibble(col[5]) << 4) + hex2nibble(col[6]) ); // B
-            break;
-          case 5:
-          case 4:
-            trns.red   = (uint16_t)( hex2nibble(col[1]) * (16 + 1) ); // R
-            trns.green = (uint16_t)( hex2nibble(col[2]) * (16 + 1) ); // G
-            trns.blue  = (uint16_t)( hex2nibble(col[3]) * (16 + 1) ); // B
-            break;
-          default:
-            error("TRNS colour not understood: '%s'", col);
-          }
-          has_trns = 1;
-        } else {
-          int idx = hash_color((const unsigned char *)col);
-          if (idx < 0 || idx > 658 || memcmp(col, col_name[idx], 2) != 0) {
-            error("Not a valid colour name: %s", col);
-          }
-          uint8_t *vals = col_int[idx];
-          trns.red   = vals[0];
-          trns.green = vals[1];
-          trns.blue  = vals[2];
-        }
-      } else if (isInteger(trns_) && length(trns_) == 3) {
+        uint32_t icol = col_to_int(col);
+        trns.red   = (uint16_t)CF_RED(icol);
+        trns.green = (uint16_t)CF_GREEN(icol);
+        trns.blue  = (uint16_t)CF_BLUE(icol);
+        has_trns = 1;
+      } else if (Rf_isInteger(trns_) && Rf_length(trns_) == 3) {
         // Rprintf("Integer vector for trns\n");
         int *ptr = INTEGER(trns_);
         trns.red   = (uint16_t)ptr[0]; // R
         trns.green = (uint16_t)ptr[1]; // G
         trns.blue  = (uint16_t)ptr[2]; // B
         has_trns = 1;
-      } else if (isReal(trns_) && length(trns_) == 3) {
+      } else if (Rf_isReal(trns_) && Rf_length(trns_) == 3) {
         // Rprintf("Real vector for trns\n");
         double *ptr = REAL(trns_);
         trns.red   = (uint16_t)ptr[0]; // R
@@ -169,9 +136,10 @@ SEXP write_png_core_(void *image, size_t nbytes, uint32_t width, uint32_t height
         trns.blue  = (uint16_t)ptr[2]; // B
         has_trns = 1;
       } else {
-        warning("Unknown argument for 'trns' %s\n", type2char((SEXPTYPE)TYPEOF(trns_)));
+      //   Rf_warning("Unknown argument for 'trns' %s\n", type2char((SEXPTYPE)TYPEOF(trns_)));
       }
     } else {
+      // has_trns = 0
       // Rprintf("trns given and --NOT-- valid for image type\n");
     }
     
@@ -179,7 +147,7 @@ SEXP write_png_core_(void *image, size_t nbytes, uint32_t width, uint32_t height
     if (has_trns) {
       err = spng_set_trns(ctx, &trns);
       if (err) {
-        warning("spng_encode_image() TRNS error: %s\n", spng_strerror(err));
+        Rf_warning("spng_encode_image() TRNS error: %s\n", spng_strerror(err));
       }
     }
     
@@ -191,7 +159,7 @@ SEXP write_png_core_(void *image, size_t nbytes, uint32_t width, uint32_t height
   //   tRNS CHunk for alpha channel
   //   PLTE Chunk for RGB palette
   //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
-  if (!isNull(palette_)) {
+  if (!Rf_isNull(palette_)) {
     
     // struct spng_trns
     // {
@@ -209,51 +177,16 @@ SEXP write_png_core_(void *image, size_t nbytes, uint32_t width, uint32_t height
     struct spng_trns trns;
     struct spng_plte plte;
     
-    plte.n_entries = (uint32_t)length(palette_); // length is checked prior to calling core func
-    trns.n_type3_entries = (uint32_t)length(palette_); // always match palette length
+    plte.n_entries = (uint32_t)Rf_length(palette_); // length is checked prior to calling core func
+    trns.n_type3_entries = (uint32_t)Rf_length(palette_); // always match palette length
     
-    for (int i = 0; i < length(palette_); i++) {
+    for (int i = 0; i < Rf_length(palette_); i++) {
       const char *col = CHAR(STRING_ELT(palette_, i));
-      if (col[0] == '#') {
-        switch (strlen(col)) {
-        case 9:
-          plte.entries[i].red   = (uint8_t)( (hex2nibble(col[1]) << 4) + hex2nibble(col[2]) ); // R
-          plte.entries[i].green = (uint8_t)( (hex2nibble(col[3]) << 4) + hex2nibble(col[4]) ); // G
-          plte.entries[i].blue  = (uint8_t)( (hex2nibble(col[5]) << 4) + hex2nibble(col[6]) ); // B
-          trns.type3_alpha[i]   = (uint8_t)( (hex2nibble(col[7]) << 4) + hex2nibble(col[8]) ); // A
-          break;
-        case 7:
-          plte.entries[i].red   = (uint8_t)( (hex2nibble(col[1]) << 4) + hex2nibble(col[2]) ); // R
-          plte.entries[i].green = (uint8_t)( (hex2nibble(col[3]) << 4) + hex2nibble(col[4]) ); // G
-          plte.entries[i].blue  = (uint8_t)( (hex2nibble(col[5]) << 4) + hex2nibble(col[6]) ); // B
-          trns.type3_alpha[i]   = 255; // opaque
-          break;
-        case 4:
-          plte.entries[i].red   = (uint8_t)( hex2nibble(col[1]) * (16 + 1)); // R
-          plte.entries[i].green = (uint8_t)( hex2nibble(col[2]) * (16 + 1)); // G
-          plte.entries[i].blue  = (uint8_t)( hex2nibble(col[3]) * (16 + 1)); // B
-          trns.type3_alpha[i]   = (uint8_t)( hex2nibble(col[4]) * (16 + 1)); // A
-          break;
-        case 3:
-          plte.entries[i].red   = (uint8_t)( hex2nibble(col[1]) * (16 + 1)); // R
-          plte.entries[i].green = (uint8_t)( hex2nibble(col[2]) * (16 + 1)); // G
-          plte.entries[i].blue  = (uint8_t)( hex2nibble(col[3]) * (16 + 1)); // B
-          trns.type3_alpha[i]   = 255; // opaque
-          break;
-        default:
-          error("Unknown hex colour '%s'", col);
-        }
-      } else {
-        int idx = hash_color((const unsigned char *)col);
-        if (idx < 0 || idx > 658 || memcmp(col, col_name[idx], 2) != 0) {
-          error("Not a valid colour name: %s", col);
-        }
-        uint8_t *vals = col_int[idx];
-        plte.entries[i].red   = vals[0]; // R
-        plte.entries[i].green = vals[1]; // G
-        plte.entries[i].blue  = vals[2]; // B
-        trns.type3_alpha[i]   = 255; // opaque
-      }
+      uint32_t icol = col_to_int(col);
+      plte.entries[i].red   = CF_RED(icol);
+      plte.entries[i].green = CF_GREEN(icol);
+      plte.entries[i].blue  = CF_BLUE(icol);
+      trns.type3_alpha[i]   = CF_ALPHA(icol);
     }
     
     // Set PLTE chunk
@@ -262,7 +195,7 @@ SEXP write_png_core_(void *image, size_t nbytes, uint32_t width, uint32_t height
       if (fp) fclose(fp);
       if (free_image_on_error) free(image);
       spng_ctx_free(ctx);
-      error("spng_encode_image() PLTE error: %s\n", spng_strerror(err));
+      Rf_error("spng_encode_image() PLTE error: %s\n", spng_strerror(err));
     }
     
     // Set tRNS chunk
@@ -271,7 +204,7 @@ SEXP write_png_core_(void *image, size_t nbytes, uint32_t width, uint32_t height
       if (fp) fclose(fp);
       if (free_image_on_error) free(image);
       spng_ctx_free(ctx);
-      error("spng_encode_image() TRNS error: %s\n", spng_strerror(err));
+      Rf_error("spng_encode_image() TRNS error: %s\n", spng_strerror(err));
     }
     
     err = spng_encode_chunks(ctx);
@@ -279,7 +212,7 @@ SEXP write_png_core_(void *image, size_t nbytes, uint32_t width, uint32_t height
       if (fp) fclose(fp);
       if (free_image_on_error) free(image);
       spng_ctx_free(ctx);
-      error("spng_encode_image() chunks error: %s\n", spng_strerror(err));
+      Rf_error("spng_encode_image() chunks error: %s\n", spng_strerror(err));
     }
   }
   
@@ -296,14 +229,14 @@ SEXP write_png_core_(void *image, size_t nbytes, uint32_t width, uint32_t height
     if (fp) fclose(fp);
     if (free_image_on_error) free(image);
     spng_ctx_free(ctx);
-    error("spng_encode_image() error: %s\n", spng_strerror(err));
+    Rf_error("spng_encode_image() error: %s\n", spng_strerror(err));
   }
   
   
   //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
   // IF writing to file, can now just return to the caller
   //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
-  if (!isNull(file_)) {
+  if (!Rf_isNull(file_)) {
     spng_ctx_free(ctx);
     fclose(fp);
     return R_NilValue;
@@ -319,13 +252,13 @@ SEXP write_png_core_(void *image, size_t nbytes, uint32_t width, uint32_t height
   if(png_buf == NULL) {
     if (free_image_on_error) free(image);
     spng_ctx_free(ctx);
-    error("spng_get_png_buffer() error: %s\n", spng_strerror(err));
+    Rf_error("spng_get_png_buffer() error: %s\n", spng_strerror(err));
   }
   
   //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
   // Copy into R raw vector
   //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
-  SEXP res_ = PROTECT(allocVector(RAWSXP, (R_xlen_t)png_size));
+  SEXP res_ = PROTECT(Rf_allocVector(RAWSXP, (R_xlen_t)png_size));
   memcpy(RAW(res_), png_buf, png_size);
   
   //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
@@ -353,8 +286,8 @@ SEXP write_png_from_raw_vec_(SEXP image_, SEXP file_, SEXP use_filter_,
   //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
   // Unpack the 'raw_spec' list into variables
   //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
-  if (isNull(raw_spec_) || TYPEOF(raw_spec_) != VECSXP || length(raw_spec_) < 4) {
-    error("'raw_spec' must be a named list with 4 elements");
+  if (Rf_isNull(raw_spec_) || TYPEOF(raw_spec_) != VECSXP || Rf_length(raw_spec_) < 4) {
+    Rf_error("'raw_spec' must be a named list with 4 elements");
   }
   
   uint32_t width  = 0;
@@ -362,26 +295,26 @@ SEXP write_png_from_raw_vec_(SEXP image_, SEXP file_, SEXP use_filter_,
   uint32_t depth  = 0;
   uint32_t bits   = 0;
   
-  SEXP nms_ = getAttrib(raw_spec_, R_NamesSymbol);
-  if (isNull(nms_) || length(nms_) != length(raw_spec_)) {
-    error("'raw_spec' must be a named list with 4 elements.");
+  SEXP nms_ = Rf_getAttrib(raw_spec_, R_NamesSymbol);
+  if (Rf_isNull(nms_) || Rf_length(nms_) != Rf_length(raw_spec_)) {
+    Rf_error("'raw_spec' must be a named list with 4 elements.");
   }
   
-  for (int i = 0; i < length(nms_); i++) {
+  for (int i = 0; i < Rf_length(nms_); i++) {
     const char *nm = CHAR(STRING_ELT(nms_, i));
     if (strcmp(nm, "width") == 0) {
-      width = (uint32_t)asInteger(VECTOR_ELT(raw_spec_, i));
+      width = (uint32_t)Rf_asInteger(VECTOR_ELT(raw_spec_, i));
     } else if (strcmp(nm, "height") == 0) {
-      height = (uint32_t)asInteger(VECTOR_ELT(raw_spec_, i));
+      height = (uint32_t)Rf_asInteger(VECTOR_ELT(raw_spec_, i));
     } else if (strcmp(nm, "depth") == 0) {
-      depth = (uint32_t)asInteger(VECTOR_ELT(raw_spec_, i));
+      depth = (uint32_t)Rf_asInteger(VECTOR_ELT(raw_spec_, i));
     } else if (strcmp(nm, "bits") == 0) {
-      bits = (uint32_t)asInteger(VECTOR_ELT(raw_spec_, i));
+      bits = (uint32_t)Rf_asInteger(VECTOR_ELT(raw_spec_, i));
     }
   }
   
   if (width == 0 || height == 0 || depth == 0 || bits == 0) {
-    error("'raw_spec' must contain 'width', 'height', 'depth', 'bits'");
+    Rf_error("'raw_spec' must contain 'width', 'height', 'depth', 'bits'");
   }
   
   
@@ -394,9 +327,9 @@ SEXP write_png_from_raw_vec_(SEXP image_, SEXP file_, SEXP use_filter_,
     size *= 2; 
   }
   
-  if (size != length(image_)) {
-    error("Mismatch between length of raw vector (%i) and raw_spec (%i x %i x %i x %i/8)", 
-          length(image_), width, height, depth, bits);
+  if (size != Rf_length(image_)) {
+    Rf_error("Mismatch between length of raw vector (%i) and raw_spec (%i x %i x %i x %i/8)", 
+          Rf_length(image_), width, height, depth, bits);
   }
   
   enum spng_color_type color_type = SPNG_COLOR_TYPE_TRUECOLOR_ALPHA;
@@ -415,11 +348,11 @@ SEXP write_png_from_raw_vec_(SEXP image_, SEXP file_, SEXP use_filter_,
     color_type = SPNG_COLOR_TYPE_GRAYSCALE;
     break;
   default:
-    error("Depth not understood: %i", depth);
+    Rf_error("Depth not understood: %i", depth);
   }
   
   return write_png_core_(
-    RAW(image_), (size_t)length(image_), width, height, file_,
+    RAW(image_), (size_t)Rf_length(image_), width, height, file_,
     color_type,
     R_NilValue, // Palette
     use_filter_, compression_level_,
@@ -441,8 +374,8 @@ SEXP write_png_from_nara_(SEXP nara_, SEXP file_, SEXP use_filter_, SEXP compres
   // Options
   //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
   void *image = (void *)INTEGER(nara_);
-  size_t nbytes = (size_t)(length(nara_) * 4.0);
-  SEXP dims_ = getAttrib(nara_, R_DimSymbol);
+  size_t nbytes = (size_t)(Rf_length(nara_) * 4.0);
+  SEXP dims_ = Rf_getAttrib(nara_, R_DimSymbol);
   uint32_t width  = (uint32_t)INTEGER(dims_)[1];
   uint32_t height = (uint32_t)INTEGER(dims_)[0];
   
@@ -466,8 +399,8 @@ SEXP write_png_from_raster_(SEXP ras_, SEXP file_, SEXP use_filter_, SEXP compre
   //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
   // Options
   //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
-  size_t nbytes   = (size_t)(length(ras_) * 4.0);
-  SEXP dims_      = getAttrib(ras_, R_DimSymbol);
+  size_t nbytes   = (size_t)(Rf_length(ras_) * 4.0);
+  SEXP dims_      = Rf_getAttrib(ras_, R_DimSymbol);
   uint32_t width  = (uint32_t)INTEGER(dims_)[1];
   uint32_t height = (uint32_t)INTEGER(dims_)[0];
   
@@ -475,50 +408,14 @@ SEXP write_png_from_raster_(SEXP ras_, SEXP file_, SEXP use_filter_, SEXP compre
   // Convert from raster to image
   //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
   unsigned char *image = (unsigned char *)malloc(nbytes);
+  
   if (image == NULL) {
-    error("Could not allocate image buffer");
+    Rf_error("Could not allocate image buffer");
   }
-  unsigned char *im_ptr = image;
-  for (int i = 0; i < Rf_xlength(ras_); i++) {
+  uint32_t *im_ptr = (uint32_t *)image;
+  for (int i = 0; i < Rf_length(ras_); i++) {
     const char *col = CHAR(STRING_ELT(ras_, i));
-    if (col[0] == '#') {
-      switch(strlen(col)) {
-      case 9:
-        *im_ptr++ = (unsigned char)( (hex2nibble(col[1]) << 4) + hex2nibble(col[2]) ); // R
-        *im_ptr++ = (unsigned char)( (hex2nibble(col[3]) << 4) + hex2nibble(col[4]) ); // G
-        *im_ptr++ = (unsigned char)( (hex2nibble(col[5]) << 4) + hex2nibble(col[6]) ); // B
-        *im_ptr++ = (unsigned char)( (hex2nibble(col[7]) << 4) + hex2nibble(col[8]) ); // A
-        break;
-      case 7:
-        *im_ptr++ = (unsigned char)( (hex2nibble(col[1]) << 4) + hex2nibble(col[2]) ); // R
-        *im_ptr++ = (unsigned char)( (hex2nibble(col[3]) << 4) + hex2nibble(col[4]) ); // G
-        *im_ptr++ = (unsigned char)( (hex2nibble(col[5]) << 4) + hex2nibble(col[6]) ); // B
-        *im_ptr++ = 255; // A
-        // Rprintf("%02X %02X %02X %02X\n", *(im_ptr - 4), *(im_ptr - 3), *(im_ptr - 2), *(im_ptr - 1));
-        break;
-      case 5:
-        *im_ptr++ = (unsigned char)( hex2nibble(col[1]) * (16 + 1) ); // R
-        *im_ptr++ = (unsigned char)( hex2nibble(col[2]) * (16 + 1) ); // G
-        *im_ptr++ = (unsigned char)( hex2nibble(col[3]) * (16 + 1) ); // B
-        *im_ptr++ = (unsigned char)( hex2nibble(col[4]) * (16 + 1) ); // A
-        break;
-      case 4:
-        *im_ptr++ = (unsigned char)( hex2nibble(col[1]) * (16 + 1) ); // R
-        *im_ptr++ = (unsigned char)( hex2nibble(col[2]) * (16 + 1) ); // G
-        *im_ptr++ = (unsigned char)( hex2nibble(col[3]) * (16 + 1) ); // B
-        *im_ptr++ = 255; // A
-        break;
-      default:
-        error("hex colour in raster not understood: '%s'", col);
-      }
-    } else {
-      int idx = hash_color((const unsigned char *)col);
-      if (idx < 0 || idx > 658 || memcmp(col, col_name[idx], 2) != 0) {
-        error("Not a valid colour name: %s", col);
-      }
-      memcpy(im_ptr, col_int[idx], 4 * sizeof(uint8_t));
-      im_ptr += 4;
-    }
+    im_ptr[i] = col_to_int(col);
   }
   
   //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
@@ -553,8 +450,8 @@ SEXP write_png_from_raster_rgb_(SEXP ras_, SEXP file_, SEXP use_filter_, SEXP co
   //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
   // Options
   //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
-  size_t nbytes   = (size_t)(length(ras_) * 3.0);
-  SEXP dims_      = getAttrib(ras_, R_DimSymbol);
+  size_t nbytes   = (size_t)(Rf_length(ras_) * 3.0);
+  SEXP dims_      = Rf_getAttrib(ras_, R_DimSymbol);
   uint32_t width  = (uint32_t)INTEGER(dims_)[1];
   uint32_t height = (uint32_t)INTEGER(dims_)[0];
   
@@ -563,38 +460,15 @@ SEXP write_png_from_raster_rgb_(SEXP ras_, SEXP file_, SEXP use_filter_, SEXP co
   //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
   unsigned char *image = (unsigned char *)malloc(nbytes);
   if (image == NULL) {
-    error("Could not allocate image buffer");
+    Rf_error("Could not allocate image buffer");
   }
   unsigned char *im_ptr = image;
-  for (int i = 0; i < Rf_xlength(ras_); i++) {
+  for (int i = 0; i < Rf_length(ras_); i++) {
     const char *col = CHAR(STRING_ELT(ras_, i));
-    if (col[0] == '#') { 
-      switch(strlen(col)) {
-      case 9:
-      case 7:
-        *im_ptr++ = (unsigned char)( (hex2nibble(col[1]) << 4) + hex2nibble(col[2]) ); // R
-        *im_ptr++ = (unsigned char)( (hex2nibble(col[3]) << 4) + hex2nibble(col[4]) ); // G
-        *im_ptr++ = (unsigned char)( (hex2nibble(col[5]) << 4) + hex2nibble(col[6]) ); // B
-        break;
-      case 5:
-      case 4:
-        *im_ptr++ = (unsigned char)( hex2nibble(col[1]) * (16 + 1) ); // R
-        *im_ptr++ = (unsigned char)( hex2nibble(col[2]) * (16 + 1) ); // G
-        *im_ptr++ = (unsigned char)( hex2nibble(col[3]) * (16 + 1) ); // B
-        break;
-      default:
-        error("write_png_from_raster_rgb_(): hex colour not understood: '%s'", col);
-      }
-    } else {
-      int idx = hash_color((const unsigned char *)col);
-      if (idx < 0 || idx > 658 || memcmp(col, col_name[idx], 2) != 0) {
-        error("Not a valid colour name: %s", col);
-      }
-      uint8_t *vals = col_int[idx];
-      *im_ptr++ = *vals++;
-      *im_ptr++ = *vals++;
-      *im_ptr++ = *vals++;
-    }
+    uint32_t icol = col_to_int(col);
+    *im_ptr++ = CF_RED(icol);
+    *im_ptr++ = CF_GREEN(icol);
+    *im_ptr++ = CF_BLUE(icol);
   }
   
   //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
@@ -629,17 +503,17 @@ SEXP write_png_from_array_(SEXP arr_, SEXP file_, SEXP use_filter_, SEXP compres
   //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
   // Options
   //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
-  size_t nbytes = (size_t)(length(arr_));
+  size_t nbytes = (size_t)(Rf_length(arr_));
   
   enum spng_color_type fmt;
   int nchannels;
-  SEXP dims_ = getAttrib(arr_, R_DimSymbol);
-  if (length(dims_) == 2) {
+  SEXP dims_ = Rf_getAttrib(arr_, R_DimSymbol);
+  if (Rf_length(dims_) == 2) {
     fmt = SPNG_COLOR_TYPE_GRAYSCALE;
     nchannels = 1;
     // SPNG_COLOR_TYPE_TRUECOLOR_ALPHA
-    // error("Must be 3d array");
-  } else if (length(dims_) == 3) {
+    // Rf_error("Must be 3d array");
+  } else if (Rf_length(dims_) == 3) {
     nchannels = INTEGER(dims_)[2];
     switch(nchannels) {
     case 2:
@@ -652,10 +526,10 @@ SEXP write_png_from_array_(SEXP arr_, SEXP file_, SEXP use_filter_, SEXP compres
       fmt = SPNG_COLOR_TYPE_TRUECOLOR_ALPHA;
       break;
     default:
-      error("Unknown 3rd dimension length: %i", INTEGER(dims_)[2]);
+      Rf_error("Unknown 3rd dimension length: %i", INTEGER(dims_)[2]);
     }
   } else {
-    error("Unknown dims length: %i", length(dims_));
+    Rf_error("Unknown dims length: %i", Rf_length(dims_));
   }
   uint32_t width  = (uint32_t)INTEGER(dims_)[1];
   uint32_t height = (uint32_t)INTEGER(dims_)[0];
@@ -665,15 +539,15 @@ SEXP write_png_from_array_(SEXP arr_, SEXP file_, SEXP use_filter_, SEXP compres
   //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
   unsigned char *image = (unsigned char *)malloc(nbytes);
   if (image == NULL) {
-    error("Could not allocate image buffer");
+    Rf_error("Could not allocate image buffer");
   }
   
   int npixels = (int)(width * height);
   unsigned char *im_ptr = image;
   
-  if (isReal(arr_)) {
+  if (Rf_isReal(arr_)) {
     double *arr_ptr = REAL(arr_);
-    if (fmt == SPNG_COLOR_TYPE_GRAYSCALE && asLogical(avoid_transpose_)) {
+    if (fmt == SPNG_COLOR_TYPE_GRAYSCALE && Rf_asLogical(avoid_transpose_)) {
       double *r = arr_ptr;
       for (int idx = 0; idx < npixels; idx ++) {
         *im_ptr++ = (uint8_t)(*r++ * 255.0 + 0.5);
@@ -701,9 +575,9 @@ SEXP write_png_from_array_(SEXP arr_, SEXP file_, SEXP use_filter_, SEXP compres
         }
       }
     } 
-  } else if (isInteger(arr_)) {
+  } else if (Rf_isInteger(arr_)) {
     int32_t *arr_ptr = INTEGER(arr_);
-    if (fmt == SPNG_COLOR_TYPE_GRAYSCALE && asLogical(avoid_transpose_)) {
+    if (fmt == SPNG_COLOR_TYPE_GRAYSCALE && Rf_asLogical(avoid_transpose_)) {
       int32_t *r = arr_ptr;
       for (int idx = 0; idx < npixels; idx ++) {
         *im_ptr++ = (uint8_t)(*r++);
@@ -763,22 +637,22 @@ SEXP write_png_from_array_(SEXP arr_, SEXP file_, SEXP use_filter_, SEXP compres
 SEXP write_png_from_array16_(SEXP arr_, SEXP file_, SEXP use_filter_, SEXP compression_level_, 
                              SEXP avoid_transpose_) {
   
-  // Rprintf("write_png_from_array16_()  %i\n", isReal(arr_));
+  // Rprintf("write_png_from_array16_()  %i\n", Rf_isReal(arr_));
   
   //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
   // Options
   //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
-  size_t nbytes = (size_t)(2.0 * length(arr_));
+  size_t nbytes = (size_t)(2.0 * Rf_length(arr_));
   
   enum spng_color_type fmt;
-  SEXP dims_ = getAttrib(arr_, R_DimSymbol);
+  SEXP dims_ = Rf_getAttrib(arr_, R_DimSymbol);
   int nchannels;
-  if (length(dims_) == 2) {
+  if (Rf_length(dims_) == 2) {
     fmt = SPNG_COLOR_TYPE_GRAYSCALE;
     nchannels = 1;
     // SPNG_COLOR_TYPE_TRUECOLOR_ALPHA
-    // error("Must be 3d array");
-  } else if (length(dims_) == 3) {
+    // Rf_error("Must be 3d array");
+  } else if (Rf_length(dims_) == 3) {
     nchannels = INTEGER(dims_)[2];
     switch(nchannels) {
     case 2:
@@ -791,10 +665,10 @@ SEXP write_png_from_array16_(SEXP arr_, SEXP file_, SEXP use_filter_, SEXP compr
       fmt = SPNG_COLOR_TYPE_TRUECOLOR_ALPHA;
       break;
     default:
-      error("Unknown 3rd dimension length: %i", INTEGER(dims_)[2]);
+      Rf_error("Unknown 3rd dimension length: %i", INTEGER(dims_)[2]);
     }
   } else {
-    error("Unknown dims length: %i", length(dims_));
+    Rf_error("Unknown dims length: %i", Rf_length(dims_));
   }
   uint32_t width  = (uint32_t)INTEGER(dims_)[1];
   uint32_t height = (uint32_t)INTEGER(dims_)[0];
@@ -804,15 +678,15 @@ SEXP write_png_from_array16_(SEXP arr_, SEXP file_, SEXP use_filter_, SEXP compr
   //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
   uint16_t *image = (uint16_t *)malloc(nbytes);
   if (image == NULL) {
-    error("Could not allocate image buffer");
+    Rf_error("Could not allocate image buffer");
   }
   
   int npixels = (int)(width * height);
   uint16_t *im_ptr = image;
   
-  if (isReal(arr_)) {
+  if (Rf_isReal(arr_)) {
     double *arr_ptr = REAL(arr_);
-    if (fmt == SPNG_COLOR_TYPE_GRAYSCALE && asLogical(avoid_transpose_)) {
+    if (fmt == SPNG_COLOR_TYPE_GRAYSCALE && Rf_asLogical(avoid_transpose_)) {
       double *r = arr_ptr;
       for (int idx = 0; idx < npixels; idx ++) {
         *im_ptr++ = (uint16_t)(*r++ * 65535.0 + 0.5);
@@ -840,9 +714,9 @@ SEXP write_png_from_array16_(SEXP arr_, SEXP file_, SEXP use_filter_, SEXP compr
         }
       }
     } 
-  } else if (isInteger(arr_)) {
+  } else if (Rf_isInteger(arr_)) {
     int32_t *arr_ptr = INTEGER(arr_);
-    if (fmt == SPNG_COLOR_TYPE_GRAYSCALE && asLogical(avoid_transpose_)) {
+    if (fmt == SPNG_COLOR_TYPE_GRAYSCALE && Rf_asLogical(avoid_transpose_)) {
       int32_t *r = arr_ptr;
       for (int idx = 0; idx < npixels; idx ++) {
         *im_ptr++ = (uint16_t)(*r++);
@@ -905,12 +779,12 @@ SEXP write_png_indexed_(SEXP arr_, SEXP file_, SEXP palette_, SEXP use_filter_,
   //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
   // Options
   //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
-  size_t nbytes = (size_t)(length(arr_));
+  size_t nbytes = (size_t)(Rf_length(arr_));
   
   enum spng_color_type fmt = SPNG_COLOR_TYPE_INDEXED;
-  SEXP dims_ = getAttrib(arr_, R_DimSymbol);
-  if (length(dims_) != 2) {
-    error("write_png_indexed_(): Must be 2-D array");
+  SEXP dims_ = Rf_getAttrib(arr_, R_DimSymbol);
+  if (Rf_length(dims_) != 2) {
+    Rf_error("write_png_indexed_(): Must be 2-D array");
   } 
   uint32_t width  = (uint32_t)INTEGER(dims_)[1];
   uint32_t height = (uint32_t)INTEGER(dims_)[0];
@@ -920,14 +794,14 @@ SEXP write_png_indexed_(SEXP arr_, SEXP file_, SEXP palette_, SEXP use_filter_,
   //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
   unsigned char *image = (unsigned char *)malloc(nbytes);
   if (image == NULL) {
-    error("Could not allocate image buffer");
+    Rf_error("Could not allocate image buffer");
   }
   
   int npixels = (int)(width * height);
   unsigned char *im_ptr = image;
   
-  if (isInteger(arr_)) {
-    if (asLogical(avoid_transpose_)) {
+  if (Rf_isInteger(arr_)) {
+    if (Rf_asLogical(avoid_transpose_)) {
       int32_t *r = INTEGER(arr_);
       for (int idx = 0; idx < npixels; idx ++) {
         *im_ptr++ = (unsigned char)(*r++);
@@ -942,8 +816,8 @@ SEXP write_png_indexed_(SEXP arr_, SEXP file_, SEXP palette_, SEXP use_filter_,
         }
       }
     }
-  } else if (isReal(arr_)) {
-    if (asLogical(avoid_transpose_)) {
+  } else if (Rf_isReal(arr_)) {
+    if (Rf_asLogical(avoid_transpose_)) {
       double *r = REAL(arr_);
       for (int idx = 0; idx < npixels; idx ++) {
         *im_ptr++ = (unsigned char)(*r++);
@@ -959,13 +833,13 @@ SEXP write_png_indexed_(SEXP arr_, SEXP file_, SEXP palette_, SEXP use_filter_,
       }
     }
   } else {
-    error("Index type not understood");
+    Rf_error("Index type not understood");
   }
   
   //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
   // No tranposition, so swap width/height value
   //~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
-  if (asLogical(avoid_transpose_)) {
+  if (Rf_asLogical(avoid_transpose_)) {
     uint32_t tmp = height;
     height = width;
     width = tmp;
@@ -1002,30 +876,30 @@ SEXP write_png_(SEXP image_, SEXP file_, SEXP palette_, SEXP use_filter_,
                 SEXP compression_level_, SEXP avoid_transpose_, SEXP bits_,
                 SEXP trns_, SEXP raw_spec_) {
   
-  if (!isNull(palette_)) {
-    if (!isMatrix(image_)) {
-      error("write_png(): When palette provided image must be a matrix.");
+  if (!Rf_isNull(palette_)) {
+    if (!Rf_isMatrix(image_)) {
+      Rf_error("write_png(): When palette provided image must be a matrix.");
     }
-    if (!isNumeric(image_)) {
-      error("write_png(): When writing paletted PNG, image must be integer or numeric matrix with values in range [0, 255]");
+    if (!Rf_isNumeric(image_)) {
+      Rf_error("write_png(): When writing paletted PNG, image must be integer or numeric matrix with values in range [0, 255]");
     }
-    if (length(palette_) > 256 || TYPEOF(palette_) != STRSXP) {
-      error("Palette must be a character vector of hex colors. length <= 256 elements");
+    if (Rf_length(palette_) > 256 || TYPEOF(palette_) != STRSXP) {
+      Rf_error("Palette must be a character vector of hex colors. length <= 256 elements");
     }
     return write_png_indexed_(image_, file_, palette_, use_filter_, compression_level_, avoid_transpose_);
-  } else if (inherits(image_, "nativeRaster")) {
+  } else if (Rf_inherits(image_, "nativeRaster")) {
     return write_png_from_nara_(image_, file_, use_filter_, compression_level_);
-  } else if (inherits(image_, "raster") && TYPEOF(image_) == STRSXP) {
-    if (length(image_) == 0) {
-      error("Zero length rasters not valid");
+  } else if (Rf_inherits(image_, "raster") && TYPEOF(image_) == STRSXP) {
+    if (Rf_length(image_) == 0) {
+      Rf_error("Zero length rasters not valid");
     }
-    if (isNull(trns_)) {
+    if (Rf_isNull(trns_)) {
       return write_png_from_raster_(image_, file_, use_filter_, compression_level_);
     } else {
       return write_png_from_raster_rgb_(image_, file_, use_filter_, compression_level_, trns_);
     } 
-  } else if ((isArray(image_) || isMatrix(image_)) && (isReal(image_) || isInteger(image_))) {
-    if (asInteger(bits_) == 16) {
+  } else if ((Rf_isArray(image_) || Rf_isMatrix(image_)) && (Rf_isReal(image_) || Rf_isInteger(image_))) {
+    if (Rf_asInteger(bits_) == 16) {
       return write_png_from_array16_(image_, file_, use_filter_, compression_level_, avoid_transpose_);
     } else {
       return write_png_from_array_(image_, file_, use_filter_, compression_level_, avoid_transpose_, trns_);
@@ -1035,7 +909,7 @@ SEXP write_png_(SEXP image_, SEXP file_, SEXP palette_, SEXP use_filter_,
                                    raw_spec_);
   }
   
-  error("write_png(): R image container not understood");
+  Rf_error("write_png(): R image container not understood");
   return R_NilValue;
 }
 
